const gulp = require('gulp')

const uglify = require('gulp-uglify')
const concat = require('gulp-concat')

const jshint = require('gulp-jshint')

const less = require('gulp-less')
const minifyCSS = require('gulp-cssnano')
const prefixer = require('gulp-autoprefixer')

const bSync = require('browser-sync')

const wiredep = require('wiredep').stream
const mainBowerFiles = require('main-bower-files')

const del = require('del')

gulp.task('clean', function() {
    return del(['dist'])
})

gulp.task('test', function() {
    return gulp.src('./src/js/*.js')
        .pipe(jshint())
        .pipe(jshint.reporter('default'))
        .pipe(jshint.reporter('fail'))
})

gulp.task('html', function () {
	return gulp.src('src/**/*.html')
		.pipe(gulp.dest('dist/'))
})

gulp.task('scripts',
    gulp.series('test', function scriptsInternal() {
    		let glob = mainBowerFiles('**/*.js')
    		glob.push('./src/js/**/*.js')
        return gulp.src(glob)
            .pipe(concat('main.min.js'))
            .pipe(uglify())
            .pipe(gulp.dest('dist/js'))
    })
)

gulp.task('styles', function() {
    return gulp.src('src/css/**/*.less')
        .pipe(less())
        .pipe(minifyCSS())
        .pipe(prefixer())
        .pipe(gulp.dest('dist/css'))
})

gulp.task('server', function(done) {
    bSync({
        server: {
            baseDir: ['dist', 'app']
        }
    })

    done()
})

gulp.task('default',
    gulp.series(
        'clean',
        gulp.parallel('html', 'scripts', 'styles'),
        'server',
        function watcher(done) {
        	gulp.watch('./src/**/*.html', gulp.parallel('html'))
        	gulp.watch('./src/js/**/*.js', gulp.parallel('scripts'))
        	gulp.watch('src/css/**/*.less', gulp.parallel('styles'))
        	gulp.watch('dist/**/*', bSync.reload)
        	done()
        }))
